/**************************************************************************//**
*   @file   ad7176_2.c
*   @brief  Driver file for ad7176_2
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
/*****************************************************************************/
/************************ Constants Definitions ******************************/
/*****************************************************************************/
#define AD7176_2_INIT

/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include "xil_io.h"
#include "xparameters.h"
#include "ad7176_2.h"

/*****************************************************************************
* @brief Writes data to a register
*
* @param regAddr - Register address
* @param txData - Data to be written
* @param regSize - Register size
*
* @return None
******************************************************************************/
void AD7176_2_WriteReg(int regAddr, int txData, int regSize)
{
	int cfgValue = 0x00;

	cfgValue |= (0			<< (AD7176_2_nWEN + 24)) 	|
				(0			<< (AD7176_2_RnW + 24))  	|
				(regAddr 	<< (AD7176_2_RA + 24))		|
				(txData 	<< (8 * (3 - regSize)));

	Xil_Out32((ADC_READER + AD7176_2_TXDATA_REG), cfgValue);
	Xil_Out32((ADC_READER + AD7176_2_CONTROL_REG), AD7176_2_TX_REQUEST);
	Xil_Out32((ADC_READER + AD7176_2_CONTROL_REG), AD7176_2_STOP_ALL);
	while((Xil_In32(ADC_READER + AD7176_2_STATUS_REG) & AD7176_2_TX_DONE) != AD7176_2_TX_DONE);
}

/*****************************************************************************
* @brief Reads data from a register
*
* @param regAddr - Register address
* @param regSize - Register size
* @param display - 1 = Print on UART, 0 = Do nothing on UART
*
* @return rxData - Read value
******************************************************************************/
int AD7176_2_ReadReg(int regAddr, int regSize, char display)
{
	int cfgValue = 0x00;
	int rxData;

	cfgValue |= (0			<< (AD7176_2_nWEN + 24))	|
				(1			<< (AD7176_2_RnW + 24))		|
				(regAddr 	<< (AD7176_2_RA + 24));

	Xil_Out32((ADC_READER + AD7176_2_TXDATA_REG), cfgValue);
	Xil_Out32((ADC_READER + AD7176_2_CONTROL_REG), AD7176_2_RX_REQUEST);
	Xil_Out32((ADC_READER + AD7176_2_CONTROL_REG), AD7176_2_STOP_ALL);
	while((Xil_In32(ADC_READER + AD7176_2_STATUS_REG) & AD7176_2_RX_DONE) != AD7176_2_RX_DONE);

	switch(regSize)
	{
		case 1:
			rxData = Xil_In32(ADC_READER + AD7176_2_RXDATA_REG) & 0x000000FF;
			break;
		case 2:
			rxData = Xil_In32(ADC_READER + AD7176_2_RXDATA_REG) & 0x0000FFFF;
			break;
		case 3:
			rxData = Xil_In32(ADC_READER + AD7176_2_RXDATA_REG) & 0x00FFFFFF;
			break;
		default:
			break;
	}

	if(display == 1)
	{
		xil_printf("Reg 0x%x = 0x%x\n\r", regAddr, rxData);
	}

	return(rxData);
}

/*****************************************************************************
* @brief Initialize a AD7176_2 Register
*
* @param reg - Register address, value, size
*
* @return None
******************************************************************************/
void AD7176_2_RegInit(st_reg reg)
{
	AD7176_2_WriteReg(reg.reg_addr, reg.reg_value, reg.reg_size);
}

/*****************************************************************************
* @brief Displays register values
*
* @param reg - Register address, value, size
*
* @return None
******************************************************************************/
void AD7176_2_RegDisplay(st_reg reg)
{
	AD7176_2_ReadReg(reg.reg_addr, reg.reg_size, 1);
}

/*****************************************************************************
* @brief Initializes AD7176_2 Registers
*
* @param None
*
* @return None
******************************************************************************/
void AD7176_2_Init(void)
{
	AD7176_2_RegInit(AD7176_2_regs[ADC_Mode_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Interface_Mode_Register]);
	AD7176_2_RegInit(AD7176_2_regs[GPIOCON_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Channel_Map0_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Channel_Map1_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Channel_Map2_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Channel_Map3_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Setup0_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Setup1_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Setup2_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Setup3_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Filter0_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Filter1_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Filter2_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Filter3_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Gain0_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Gain1_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Gain2_Register]);
	AD7176_2_RegInit(AD7176_2_regs[Gain3_Register]);
}

/*****************************************************************************
* @brief Display AD7176_2 Registers
*
* @param None
*
* @return None
******************************************************************************/
void AD7176_2_PrintRegisters(void)
{
	AD7176_2_RegDisplay(AD7176_2_regs[ADC_Mode_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Interface_Mode_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[GPIOCON_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[ID_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Channel_Map0_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Channel_Map1_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Channel_Map2_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Channel_Map3_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Setup0_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Setup1_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Setup2_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Setup3_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Filter0_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Filter1_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Filter2_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Filter3_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Gain0_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Gain1_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Gain2_Register]);
	AD7176_2_RegDisplay(AD7176_2_regs[Gain3_Register]);
}

/*****************************************************************************
* @brief Start Conversion
*
* @param None
*
* @return None
******************************************************************************/
void AD7176_2_StartConversion(void)
{
	Xil_Out32((ADC_READER + AD7176_2_CONTROL_REG), AD7176_2_CNV_START);
}

/*****************************************************************************
* @brief Stop Conversion
*
* @param None
*
* @return None
******************************************************************************/
void AD7176_2_StopConversion(void)
{
	Xil_Out32((ADC_READER + AD7176_2_CONTROL_REG), AD7176_2_STOP_ALL);
}
