/**************************************************************************//**
*   @file   ad7176_2.h
*   @brief  Header file for ad7176_2
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/

/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include "xparameters.h"

#ifndef AD7176_2_H_
#define AD7176_2_H_

// AD7176-2 IP Core Base Address
#define ADC_READER					XPAR_ADC_READER_0_BASEADDR

// AD7176-2 IP Core Register Mapping
#define AD7176_2_CONTROL_REG		0x00
#define AD7176_2_TXDATA_REG			0x04
#define AD7176_2_RXDATA_REG			0x08
#define AD7176_2_STATUS_REG			0x0C

// AD7176-2 Control Register Commands
#define AD7176_2_STOP_ALL			0x00
#define AD7176_2_CNV_START			0x01
#define AD7176_2_TX_REQUEST			0x02
#define AD7176_2_RX_REQUEST			0x04

// AD7176-2 Status Register Values
#define AD7176_2_TX_DONE			0x02
#define AD7176_2_RX_DONE			0x04

// AD7176-2 IC Registers
#define AD7176_2_COMMS_REG			0x00
#define AD7176_2_ST_REG				0x00
#define AD7176_2_ADCMODE_REG		0x01
#define AD7176_2_IFMODE_REG			0x02
#define AD7176_2_REGCHECK_REG		0x03
#define AD7176_2_DATA_REG			0x04
#define AD7176_2_GPIOCON_REG		0x06
#define AD7176_2_ID_REG				0x07
#define AD7176_2_CHMAP0_REG			0x10
#define AD7176_2_CHMAP1_REG			0x11
#define AD7176_2_CHMAP2_REG			0x12
#define AD7176_2_CHMAP3_REG			0x13
#define AD7176_2_SETUPCON0			0x20
#define AD7176_2_SETUPCON1			0x21
#define AD7176_2_SETUPCON2			0x22
#define AD7176_2_SETUPCON3			0x23
#define AD7176_2_FILTCON0			0x28
#define AD7176_2_FILTCON1			0x29
#define AD7176_2_FILTCON2			0x2A
#define AD7176_2_FILTCON3			0x2B
#define AD7176_2_OFFSET0			0x30
#define AD7176_2_OFFSET1			0x31
#define AD7176_2_OFFSET2			0x32
#define AD7176_2_OFFSET3			0x33
#define AD7176_2_GAIN0				0x38
#define AD7176_2_GAIN1				0x39
#define AD7176_2_GAIN2				0x3A
#define AD7176_2_GAIN3				0x3B

// Register Specific Bits
// COMMS Register - Address 0x00
#define AD7176_2_nWEN				7
#define AD7176_2_RnW				6
#define AD7176_2_RA					0
// Status Register - Address 0x00
#define AD7176_2_nRDY				7
#define AD7176_2_ADC_ERROR			6
#define AD7176_2_CRC_ERROR			5
#define AD7176_2_REG_ERROR			4
#define AD7176_2_CHANNEL			0
// ADC Mode Register - Address 0x01
#define AD7176_2_REF_EN				15
#define AD7176_2_SING_CYC			13
#define AD7176_2_DELAY				8
#define AD7176_2_MODE				4
#define AD7176_2_CLOCKSEL			2
// Interface Mode Register - Address 0x02
#define AD7176_2_ALT_SYNC			12
#define AD7176_2_IOSTRENGTH			11
#define AD7176_2_DOUT_RESET			8
#define AD7176_2_CONTREAD			7
#define AD7176_2_DATA_STAT			6
#define AD7176_2_REG_CHECK			5
#define AD7176_2_CRC_EN				2
#define AD7176_2_WL16				0
// GPIOCON Register - Address 0x06
#define AD7176_2_MUX_IO				12
#define AD7176_2_SYNC_EN			11
#define AD7176_2_ERR_EN				9
#define AD7176_2_ERR_DAT			8
#define AD7176_2_IP_EN1				5
#define AD7176_2_IP_EN0				4
#define AD7176_2_OP_EN1				3
#define AD7176_2_OP_EN0				2
#define AD7176_2_GP_DATA1			1
#define AD7176_2_GP_DATA0			0
// Channel Map Register x - Address 0x10 - 0x13
#define AD7176_2_CH_EN0				15
#define AD7176_2_SETUP_SEL0			12
#define AD7176_2_AINPOS 			5
#define AD7176_2_AINNEG				0
// Setup Configuration Register x - Address 0x20 - 0x23
#define AD7176_2_BI_UNIPOLAR0		12
#define AD7176_2_REF_SEL0			4
// Filter Configuration Register x - Address 0x28 - 0x2B
#define AD7176_2_SIN3_MAP0			15
#define AD7176_2_ENHFILTEN0			11
#define AD7176_2_ENHFILT0			8
#define AD7176_2_ORDER0				5
#define AD7176_2_ORD0				0

enum AD7176_2_Registers
{
	Status_register = 0x00,
	ADC_Mode_Register,
	Interface_Mode_Register,
	Register_Check_Register,
	Data_Register,
	GPIOCON_Register,
	ID_Register,
	Channel_Map0_Register,
	Channel_Map1_Register,
	Channel_Map2_Register,
	Channel_Map3_Register,
	Setup0_Register,
	Setup1_Register,
	Setup2_Register,
	Setup3_Register,
	Filter0_Register,
	Filter1_Register,
	Filter2_Register,
	Filter3_Register,
	Gain0_Register,
	Gain1_Register,
	Gain2_Register,
	Gain3_Register,
	AD7176_2_REG_NO
};

typedef struct _st_reg
{
	int reg_addr;
	int reg_value;
	int reg_size;
}st_reg;

#ifdef AD7176_2_INIT
st_reg AD7176_2_regs[] =
{
		{0x00, 0x0000, 1},		// Status Register
		{0x01, 0x8000, 2},  	// ADC Mode Register
		{0x02, 0x0000, 2}, 		// Interface Mode Register
		{0x03, 0x000000, 3},  	// Reg Check Register
		{0x04, 0x000000, 3},  	// Data Register
		{0x06, 0x0800, 2},		// GPIOCON Register
		{0x07, 0x0000, 2},  	// ID Register
		{0x10, 0x8004, 2},  	// Channel Map 0 Register
		{0x11, 0x0000, 2},  	// Channel Map 1 Register
		{0x12, 0x0000, 2},  	// Channel Map 2 Register
		{0x13, 0x0000, 2},  	// Channel Map 3 Register
		{0x20, 0x1020, 2},  	// Setup Config 0 Register
		{0x21, 0x0000, 2},  	// Setup Config 1 Register
		{0x22, 0x0000, 2},  	// Setup Config 2 Register
		{0x23, 0x0000, 2},  	// Setup Config 3 Register
		{0x28, 0x0000, 2},  	// Filter Config 0 Register
		{0x29, 0x0000, 2},  	// Filter Config 1 Register
		{0x2A, 0x0000, 2},  	// Filter Config 2 Register
		{0x2B, 0x0000, 2},  	// Filter Config 3 Register
		{0x30, 0x800000, 3},  	// Offset Configuration 0 Register
		{0x31, 0x000000, 3},  	// Offset Configuration 1 Register
		{0x32, 0x000000, 3},  	// Offset Configuration 2 Register
		{0x33, 0x000000, 3},  	// Offset Configuration 3 Register
		{0x38, 0x000000, 3},  	// Gain Configuration 0 Register
		{0x39, 0x000000, 3},  	// Gain Configuration 1 Register
		{0x3A, 0x000000, 3},  	// Gain Configuration 2 Register
		{0x3B, 0x000000, 3}  	// Gain Configuration 3 Register
};
#else
extern st_reg AD7176_2_regs[AD7176_2_REG_NO];
#endif

/**************************************************************************//**
*   @file   ad7176_2.c
*   @brief  Driver file for ad7176_2
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
/*****************************************************************************/
/************************ Constants Definitions ******************************/
/*****************************************************************************/
#define AD7176_2_INIT

/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include "xil_io.h"
#include "xparameters.h"
#include "ad7176_2.h"

void AD7176_2_WriteReg(int regAddr, int txData, int regSize);
int AD7176_2_ReadReg(int regAddr, int regSize, char display);
void AD7176_2_RegInit(st_reg reg);
void AD7176_2_RegDisplay(st_reg reg);
void AD7176_2_Init(void);
void AD7176_2_PrintRegisters(void);
void AD7176_2_StartConversion(void);
void AD7176_2_StopConversion(void);

#endif /* AD7176_2_H_ */
